# -*- coding: utf-8 -*-
__author__ = 'jayin'


def is_markdown_file(filename=''):
    """
    判断给定的文件名是否是markdown文件
    """
    if filename.endswith('.md') or filename.endswith('.markdown'):
        return True
    return False


def read_file(file_path):
    """
    读取文件
    :param file_path: 文件路径
    :return: string
    """
    import codecs

    with codecs.open(file_path, mode='r', encoding='utf-8') as f:
        return f.read()


def get_markdown_p(content='', word=30):
    """
    获得第一句<p>标签的内容
    :param content: 未渲染,markdown格式内容
    :param word: 若分段失败，则截取内容的长度
    :return: string
    """
    import mistune
    from bs4 import BeautifulSoup

    md = mistune.markdown(content)
    soup = BeautifulSoup(md)

    p = soup.find('p')

    if len(p.text.split(u'。')) > 0:
        return p.text.split(u'。')[0]
    elif len(p.text.split(u'.')) > 0:
        return p.text.split()
    return p.text[0:word]










# -*- coding: utf-8 -*-
__author__ = 'jayin'

from path import Path
import _
import os
import copy
import click

# 注意文件名不能用冒号`:`，干脆直接替换成空格!

_Chinese_Punction = [
    u'！', u'“', u'”', u'；', u'《', u'》', u'。', u'、', u'，', u'【', u'】', u'（', u'）', u'？', u'：', u':'
]
_English_Punction = [
    u'!', u'"', u'"', u';', u'<', u'>', u'.', u',', u',', u'[', u']', u'(', u')', u'?', u' ', u' '
]

# Chinese 2 English
C2E = dict(map(lambda x, y: [x, y], _Chinese_Punction, _English_Punction))


def check(path='.', _echo=True):
    """
    检查路径目录(默认是当前)下的文件名是否存在中文字符的标点
    :param path: 给定目录
    :param _echo: 是否输出log
    :return:  dist
        tpl = {
            'src': '',
            'dest': ''
        }
    """

    path = Path(path)

    result = []
    tpl = {
        'src': '',
        'dest': ''
    }

    def walk(p):
        if p.isfile() and _.is_markdown_file(p):
            # 检查->改名
            _name = p.name
            for prounction in _Chinese_Punction:
                if prounction in _name:
                    # 替换标点
                    _name = _name.replace(prounction, C2E[prounction])

            if _name != p.name:
                rename = copy.deepcopy(tpl)
                rename['src'] = p.abspath()
                rename['dest'] = os.path.join(os.path.dirname(p.abspath()), _name)
                result.append(rename)

        elif p.isdir():
            for f in p.listdir():
                walk(f)

    walk(path)

    if _echo:
        if len(result) == 0:
            click.echo('Good! Not found!')
        else:
            for x in result:
                click.echo(x['src'])

    return result


def fix(path):
    """
    修复存在中文标点的
    :param path:  给定路径
    """

    result = check(path, False)

    for x in result:
        click.echo('mv %s %s' % (x['src'], x['dest']))
        Path.move(x['src'], x['dest'])

    click.echo('Finish: fix all!')








#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages
import emanual

with open('README.md') as f:
    long_description = f.read()


setup(
    name='emanual',
    version=emanual.__version__,
    description=emanual.__doc__.strip(),
    long_description=long_description,
    url='https://github.com/EManual/EManual-CLI',
    author=emanual.__author__,
    author_email='tonjayin@gmail.name',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'click',
        'path.py',
        'pypinyin',
        'mistune',
        'beautifulsoup4'
    ],
    entry_points={
        'console_scripts' : [
            'emanual=emanual.cli:main',
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
    ],
    keywords='emanual cli',
)






# -*- coding: utf-8 -*-

from path import Path
from pypinyin import lazy_pinyin
import copy
import json
import os
import _

MODE_TREE = 'tree'
MODE_FILE = 'file'

file_tpl = {
    'mode': '',
    'name': '',
    'rname': '',
    'path': ''
}

info_tpl = {
    'files': [],
    'mode': '',
    'name': '',
    'rname': '',
    'path': ''
}

raw_md_root = Path('./markdown')
dist_root = Path('./dist')
info_dist_root = './dist/{lang}'


def check_path():
    """
    检查是否存在'./markdown' './dist'
    :return:
    """
    if not raw_md_root.exists():
        raise IOError('Not found : dir ./markdown ')

    if not dist_root.exists():
        dist_root.mkdir()


def gen_info(p):
    """
    在`./markdown`下生成info.json
    :param p: the path
    """
    info = copy.deepcopy(info_tpl)
    info['mode'] = MODE_TREE if p.isdir() else MODE_FILE
    info['name'] = ''.join(lazy_pinyin(p.name))
    info['rname'] = p.name
    # p.rename(info['name'])# 路径也变为拼音
    info['path'] = ''.join(lazy_pinyin(p.lstrip(dist_root)))
    info['mtime'] = p.mtime
    for f in p.listdir():
        _file = copy.deepcopy(file_tpl)
        _file['mode'] = MODE_TREE if f.isdir() else MODE_FILE
        _file['name'] = ''.join(lazy_pinyin(f.name))
        _file['rname'] = f.name
        # _file['path'] = info['path'] + '/' + _file['name']
        _file['path'] = os.path.join(info['path'], _file['name'])
        _file['mtime'] = f.mtime
        info['files'].append(_file)

    with open(p + '/info.json', mode='w') as f:
        f.write(json.dumps(info))
    for f in p.dirs():
        gen_info(f)


def clean_up(path='./markdown'):
    """
    清除目录下:
    1. `*.json文件`
    2. `*.*~` linux下临时文件
    3. `.DS_Store` OS X 下文件描述文件
    :param path: 清除目录
    """
    os.system("find {path} -name '*.json' | xargs rm -f ".format(path=path))
    os.system("find {path} -name '*.*~' | xargs rm -f ".format(path=path))
    os.system("find {path} -name '.DS_Store' | xargs rm -f ".format(path=path))
    print('Finish: clean up !')


def dirs_pinyin(p):
    """
    将指定的目录下得所有中文文件名转换拼音
    :param p:  给定目录
    """
    if p.isfile():
        pinyin = ''.join(lazy_pinyin(p.name))
        # 拼音先隔离dirname，因为父文件夹是最后才转换成拼音
        Path.move(p.abspath(), os.path.join(p.dirname(), pinyin))
        return
    else:
        for x in p.listdir():
            dirs_pinyin(x)
            if x.isdir():
                pinyin = ''.join(lazy_pinyin(x.name))
                Path.move(x.abspath(), os.path.join(x.dirname(), pinyin))


def create_info(lang):
    """
    创建生成info.json文件以及把中文文件名变为拼音
    :return:
    """
    clean_up()
    check_path()
    lang_info_dist_root = info_dist_root.format(lang=lang)
    p = Path(lang_info_dist_root)
    if p.exists():
        os.system('rm -fr %s' % lang_info_dist_root)
    Path.copytree(raw_md_root, p)
    gen_info(p)
    dirs_pinyin(p)
    print('Finish: generate info.json, please checkout `%s`' % p)


def dist_zip(lang):
    """
    压缩create后在dist/{lang}下的文件夹到-> `{lang}.zip`
    :param lang: 语言
    """
    cwd = os.getcwd()
    dest = os.path.join(cwd, 'dist')
    if not os.path.exists(dest):
        os.makedirs(dest)

    # the zip
    zip_file = os.path.join(dest, '%s.zip' % lang)
    if os.path.exists(zip_file):
        os.system('rm %s' % zip_file)
    cmds = [
        'cd %s' % dest,
        'zip -q -r %s.zip %s/' % (lang, lang),
    ]
    os.system(' && '.join(cmds))
    print('Finish dist')


def newsfeeds_update():
    """
    更新newsfeeds模块的信息
    :return:
    """
    # newsfeeds_root -> ./dist/newsfeeds
    check_path()
    newsfeeds_root = Path(os.path.join(dist_root, 'newsfeeds'))
    if newsfeeds_root.exists():
        os.system('rm -fr %s' % newsfeeds_root)
    clean_up(dist_root)

    Path.copytree(raw_md_root, newsfeeds_root)

    files = []

    def get_files(p):
        for f in p.listdir():
            if f.isfile() and _.is_markdown_file(f.name):
                files.append(f)
            elif f.isdir():
                get_files(f)

    def gen_page_info(files):
        tpl = {
            'name': '',
            'rname': '',
            'path': '',
            'description': ''
        }
        sigle_page = 10  # 单页数量
        files.reverse()
        for page in range(1, len(files) / sigle_page + 2):
            _data = []
            for x in files[(page - 1) * sigle_page: page * sigle_page]:
                _tmp = copy.deepcopy(tpl)
                _tmp['name'] = ''.join(lazy_pinyin(x.name))
                _tmp['rname'] = x.name
                _tmp['path'] = ''.join(lazy_pinyin(x.lstrip(dist_root)))
                _tmp['description'] = _.get_markdown_p(_.read_file(x.abspath()))
                _data.append(_tmp)
            with open(os.path.join(dist_root, '%d.json' % page), mode='w') as f:
                f.write(json.dumps(_data))

    get_files(newsfeeds_root)
    gen_page_info(files)
    dirs_pinyin(newsfeeds_root)
    print('Finish newsfeeds update!\ntotal:%d' % len(files))








# -*- coding: utf-8 -*-


from emanual import make, __version__
import click


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__)
    ctx.exit()


@click.group()
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True, help='Show the version')
def main():
    """
    Welecome to EManual CLI!
    """
    pass


@main.command('create')
@click.argument('lang', )
def create_info(lang):
    """
    Create the info.json File for Given Lang 
    """
    make.create_info(lang)


@main.command('server')
@click.option('--port', default=8000, help='port to listen')
def server(port):
    """
    Server to preview the markdown files
    """
    import os

    os.system('python -m SimpleHTTPServer %s' % port)


@main.command('dist')
@click.argument('lang', )
def dist(lang):
    """
    Distributing the markdown
    """
    make.dist_zip(lang)


@main.command('newsfeeds')
@click.argument('update', default=True)
def newsfeeds(update):
    """
    The cli for NewsFeeds module
    """
    if update:
        make.newsfeeds_update()


@main.command('filename')
@click.argument('operate', default='check')
@click.argument('path', default='.')
def filename_check(operate, path):
    """
    The utils CLI for solve filename

    $ emanual filename check [path=.]:check out if given path(defualt is current work path `.`) contains Chinese punctuation
    $ emanual filename fix  [path=.]: turn Chinese punctuation to English punctuation
    """
    import filename
    if operate == 'check':
        filename.check(path)
    if operate == 'fix':
        filename.fix(path)








